import vueReady from "./vue-ready";
import domReady from "./dom-ready";
import loadScript from "./load-script";
import loadStyleLink from "./load-style-link";
import { initSharing, registerShareModule, loadWfmModule } from "./wfm";
import strToJsVar from "./str-to-js-var";

window.$HFC_CDN_URL = window.$HFC_CDN_URL || "https://cdn.jsdelivr.net/npm";
window.$HFC_WFM_CONTAINERS = {};

initSharing()
  .then(vueReady)
  .then(domReady)
  .then(() => {
    registerShareModule("vue", window.$HFC_VUE.version, window.$HFC_VUE);
    document.querySelectorAll("[hfc]").forEach(function (elem) {
      mount(elem as HTMLTemplateElement);
    });
  })
  .catch((err: any) => {
    console.warn("faild to init hyper function component web loader");
    console.error(err);
  });

function mount(elem: HTMLTemplateElement) {
  Promise.all([
    import(/* webpackMode: "eager" */ "@hyper-function/hfc-to-vue"),
    import("vue"),
  ]).then(([hfcToVue, Vue]) => {
    const {
      compile,
      createApp,
      onMounted,
      onUnmounted,
      defineComponent,
      defineAsyncComponent,
    } = Vue;

    const template = elem.innerHTML;
    let container;
    if (elem.getAttribute("hfc")) {
      container = document.querySelector(elem.getAttribute("hfc")!);
    }

    if (!container) {
      container = document.createElement("div");
      elem.insertAdjacentElement("afterend", container);
    }

    container.setAttribute("hfc", "");

    let data: any;
    const dataJsonStr = elem.getAttribute("data-json");
    if (dataJsonStr) {
      try {
        data = JSON.parse(decodeURIComponent(dataJsonStr));
      } catch (error) {}
    }

    if (!data) {
      const dataStr = elem.getAttribute(":data");
      if (dataStr) data = strToJsVar(dataStr);
    }

    data = data || {};

    let components: Record<string, any> = {
      MountEvent: defineComponent({
        emits: ["mounted", "unmounted"],
        setup({}, { slots, emit }) {
          onMounted(() => emit("mounted"));
          onUnmounted(() => emit("unmounted"));
          return () => slots.default && slots.default();
        },
      }),
    };

    function getAsyncHfc(name: string, version: string) {
      return defineAsyncComponent({
        loader() {
          const target = "@hyper.fun/" + name + "@" + version;

          let cdnUrl = window.$HFC_CDN_URL;
          const rewrite = (window as any)[`$HFC_CDN_REWRITE_${target}`];
          if (rewrite) cdnUrl = rewrite;
          const componentUrl = `${cdnUrl}/${target}/wfm/entry.js`;

          return loadScript(componentUrl)
            .then(() => {
              loadStyleLink(`${cdnUrl}/${target}/hfc.css`);
              return loadWfmModule("@hyper.fun/" + name);
            })
            .then((comp: any) => {
              return hfcToVue.default(comp.default, true);
            })
            .catch((err) => {
              console.error(err);
              console.warn(`faild to load component: ${name} ${version} `);
              return defineComponent({});
            });
        },
      });
    }

    const hfcTagNameRe = /<(\w+-[\w-]+)\s+v([\w\.-]+)=/g;
    let _tr;
    while ((_tr = hfcTagNameRe.exec(template))) {
      const [_, name, version] = _tr;
      components[name] = getAsyncHfc(name, version);
    }

    const componentStr = elem.getAttribute("imports");
    if (componentStr) {
      componentStr
        .trim()
        .split(";")
        .forEach((item) => {
          const [alias, name, version] = item.trim().split(" ");
          if (!alias || !name || !version || version[0] !== "v") return;

          components[alias] = getAsyncHfc(name, version.slice(1));
        });
    }

    const methods: Record<string, any> = {};
    const methodStr = elem.getAttribute(":methods");
    if (methodStr) {
      const methodArr = strToJsVar(methodStr);
      if (Array.isArray(methodArr)) {
        methodArr.forEach((method) => {
          if (window[method]) methods[method] = window[method];
        });
      }
    }

    const app = createApp({
      data: () => data,
      methods,
      render: compile(template, {
        runtimeGlobalName: "$HFC_VUE",
      }),
    });

    Object.keys(components).forEach((name) => {
      app.component(name, components[name]);
    });

    app.mount(container);
    elem.remove();
  });
}
